/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package cz.muni.fi.pb162.timetrial;

import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.SortedSet;
import java.util.TreeSet;

/**
 * Implementation of timekeeping machine used to measure time in timetrial contest
 * 
 * Machine registers contestants and allows to start them in order of registration.
 * Measures time for each contestant and provides (temporary and final) time list.
 * 
 * @author misko
 */
public final class Timekeeping {

    private final Queue<Contestant> startQueue = new LinkedList<Contestant>();
    private final Map<Integer, Contestant> registred = new HashMap<Integer, Contestant>();
    private final SortedSet<Contestant> times = new TreeSet<Contestant>();

    /**
     * Registers a new contestant and adds him to starting queue
     * 
     * @param contestant registered contestant
     * @throws NullPointerException if contestant is null
     * @throws IllegalArgumentException if contestant is allready registered
     */
    public void register(Contestant contestant) {
        if (contestant == null) {
            throw new NullPointerException("argument contestant");
        }

        if (registred.containsKey(contestant.getStartNumber())) {
            throw new IllegalArgumentException("contestant allready registered");
        }
        startQueue.add(contestant);
        registred.put(contestant.getStartNumber(), contestant);
    }

    /**
     * Informs whether there is an contestant to start
     * @return true if there is somebody to start, false otherwise
     */
    public boolean hasNext() {
        return !startQueue.isEmpty();
    }

    /**
     * Gives the contestant that should prepare to start
     * 
     * @return contestant to start
     */
    public Contestant prepare() {
        return startQueue.peek();
    }

    /**
     * Starts the contestant prepared to start
     * @throws IllegalStateException if there is nobody to start
     */
    public void start() {
        if (!hasNext()) {
            throw new IllegalStateException("nobody to start");
        }
        Contestant started = startQueue.poll();
        started.start(new Date());
    }

    /**
     * Finishs the performance of the contestant computing her/his time and putting
     * her/him to (temporary) time list
     * @param startNumber start number of the contestant entering the finish
     * @return contestant entering the finish
     */
    public Contestant finish(int startNumber) {
        if (!registred.containsKey(startNumber)) {
            throw new IllegalArgumentException("unknown contestant");
        }
        Contestant finished = registred.get(startNumber);
        finished.finish(new Date());
        times.add(finished);
        return finished;
    }

    /**
     * Gives the (temporary) order of competitors' finish 
     * as an unmodifable set
     * 
     * @return unmodifable sorded collection of the contestants
     */
    public SortedSet<Contestant> getOrder() {
        return Collections.unmodifiableSortedSet(times);
    }
}
